@@ -1,11 +1,26 @@
-# Copyright 2012 Gentoo Foundation
+# Copyright 2012-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
+from __future__ import unicode_literals
+
 from portage.dep import Atom, paren_enclose, use_reduce
 from portage.eapi import _get_eapi_attrs
 from portage.exception import InvalidData
 from _emerge.Package import Package
 
+def strip_slots(dep_struct):
+	"""
+	Search dep_struct for any slot := operators and remove the
+	slot/sub-slot part, while preserving the operator. The result
+	is suitable for --changed-deps comparisons.
+	"""
+	for i, x in enumerate(dep_struct):
+		if isinstance(x, list):
+			strip_slots(x)
+		elif (isinstance(x, Atom) and
+			x.slot_operator == "=" and x.slot is not None):
+			dep_struct[i] = x.with_slot("=")
+
 def find_built_slot_operator_atoms(pkg):
 	atoms = {}
 	for k in Package._dep_keys:
@@ -65,6 +80,11 @@ def evaluate_slot_operator_equal_deps(settings, use, trees):
 	return result
 
 def _eval_deps(dep_struct, vardbs):
+	# TODO: we'd use a better || () handling, i.e. || ( A:= B:= ) with both A
+	# and B installed should record subslot on A only since the package is
+	# supposed to link against that anyway, and we have no guarantee that B
+	# has matching ABI.
+
 	for i, x in enumerate(dep_struct):
 		if isinstance(x, list):
 			_eval_deps(x, vardbs)
@@ -85,11 +105,15 @@ def _eval_deps(dep_struct, vardbs):
 						dep_struct[i] = x
 						break
 			else:
-				# this dep could not be resolved, so remove the operator
-				# (user may be using package.provided and managing rebuilds
-				# manually)
-				if x.slot:
-					x = x.with_slot(x.slot)
-				else:
-					x = x.without_slot
-				dep_struct[i] = x
+				# this dep could not be resolved, possibilities include:
+				# 1. unsatisfied branch of || () dep,
+				# 2. package.provided,
+				# 3. --nodeps.
+				#
+				# just leave it as-is for now. this does not cause any special
+				# behavior while keeping the information in vdb -- necessary
+				# e.g. for @changed-deps to work properly.
+				#
+				# TODO: make it actually cause subslot rebuilds when switching
+				# || () branches.
+				pass
