@@ -512,7 +512,6 @@ class config(object):
 				if v is not None:
 					portdir_sync = v
 
-			known_repos = frozenset(known_repos)
 			self["PORTDIR"] = portdir
 			self["PORTDIR_OVERLAY"] = portdir_overlay
 			if portdir_sync:
@@ -523,6 +522,9 @@ class config(object):
 			else:
 				self.repositories = repositories
 
+			known_repos.extend(repo.location for repo in self.repositories)
+			known_repos = frozenset(known_repos)
+
 			self['PORTAGE_REPOSITORIES'] = self.repositories.config_string()
 			self.backup_changes('PORTAGE_REPOSITORIES')
 
@@ -562,8 +564,10 @@ class config(object):
 			self.user_profile_dir = locations_manager.user_profile_dir
 
 			try:
-				packages_list = [grabfile_package(os.path.join(x, "packages"),
-					verify_eapi=True) for x in self.profiles]
+				packages_list = [grabfile_package(
+					os.path.join(x.location, "packages"),
+					verify_eapi=True, eapi=x.eapi, eapi_default=None)
+					for x in profiles_complex]
 			except IOError as e:
 				if e.errno == IsADirectory.errno:
 					raise IsADirectory(os.path.join(self.profile_path,
@@ -756,7 +760,8 @@ class config(object):
 						portage.dep.ExtendedAtomDict(dict)
 					bashrc = grabdict_package(os.path.join(profile.location,
 						"package.bashrc"), recursive=1, allow_wildcard=True,
-								allow_repo=True, verify_eapi=False)
+								allow_repo=True, verify_eapi=True,
+								eapi=profile.eapi, eapi_default=None)
 					if not bashrc:
 						continue
 
@@ -779,7 +784,7 @@ class config(object):
 
 			archlist = [grabfile(os.path.join(x, "arch.list")) \
 				for x in locations_manager.profile_and_user_locations]
-			archlist = stack_lists(archlist, incremental=1)
+			archlist = sorted(stack_lists(archlist, incremental=1))
 			self.configdict["conf"]["PORTAGE_ARCHLIST"] = " ".join(archlist)
 
 			pkgprovidedlines = [grabfile(
@@ -803,12 +808,6 @@ class config(object):
 					has_invalid_data = True
 					del pkgprovidedlines[x]
 					continue
-				if cpvr[0] == "virtual":
-					writemsg(_("Virtual package in package.provided: %s\n") % \
-						myline, noiselevel=-1)
-					has_invalid_data = True
-					del pkgprovidedlines[x]
-					continue
 			if has_invalid_data:
 				writemsg(_("See portage(5) for correct package.provided usage.\n"),
 					noiselevel=-1)
